@@ -102,7 +102,7 @@ t_access ip_allowed(t_ip_addr *ip, t_accesslist *list);
 int  parse_iplist(char *line, t_iplist **list);
 bool in_iplist(t_iplist *list, t_ip_addr *ip);
 
-int  parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *delimiter);
+int  parse_keyvalue(char *line, t_keyvalue **kvlist, char *delimiter);
 t_keyvalue *remove_keyvaluelist(t_keyvalue *list);
 
 int  parse_error_handler(char *line, t_error_handler **handlers);
